# About

Passphrase is a small, simple passphrase generator.

[![builds.sr.ht status](https://builds.sr.ht/~alva/passphrase.svg)](https://builds.sr.ht/~alva/passphrase?)

## Background

I was using Bitwarden's passphrase generator, which is good,
except it only uses English words, and is also very slow.
So I made this one. It's pretty fast
(over 6000 times faster than `bw generate -p`,)
and you can use any wordlist.
An English wordlist is included, and is the default.

## Building

The program is written in [Zig](https://ziglang.org),
so you will need the Zig compiler,
which is called `zig` in most package managers.
You can also [download a static build](https://ziglang.org/download/) if it's not packaged for you.

To build and install the program:

```shell
zig build -Drelease-fast --prefix ~/.local
```

It installs an executable file in `$PREFIX/bin`,
so that should be in your `$PATH`.

To run the tests:

```shell
zig build test
```

## Usage

Use `passphrase -h` to see the available options.
Some examples:

```shell
passphrase
demontering vestlandsturnear pacemen

passphrase -x # add a random symbol
intimacy urvriden overprest$

passphrase -d # add a random digit
folkinitiativ elefantritt felfinneri0

passphrase -c # capitalise each word
Alligator Eignarlandi Armkraft

passphrase -s _ # change the separator
afdreginni_glaskasse_opplevingi

passphrase -s '' # use an empty separator
tansprungephantasmagoriandeviancy

passphrase -w 6 # it should have 6 words
dobbeltleveranse skunkworks kubikfot veggtavlone freeridden polarimeter

passphrase -f path/to/english_words.txt # use different dictionary
likelihood impassible educational

passphrase -xdcs /
Batwomen/Velopharyngeal/Bildekort&9

passphrase -p # generate a gibberish password
0"[YtPUB5R'jBs.24\X~^<z$Xmkz/tGi
```

If the output is piped into another command,
for example `passphrase | wl-copy` to put the passphrase in the clipboard,
the ending newline is omitted, so that it can be safely pasted.
