def print_code(diffs):
    print("pub fn toUpper(cp: u21) u21 {")
    print("    return switch(cp) {")
    for (k, v) in diffs:
        v.sort()
        is_neg = k < 0
        op = '+' if is_neg else '-'
        if len(v) == 1:
            print(f"        0x{v[0]:04x} => cp {op} 0x{abs(k):01x},")
        else:
            print(f"        0x{v[0]:04x}...0x{v[-1]:04x} => cp {op} 0x{abs(k):01x},")
    print("        else => cp,")
    print("    };")
    print("}")


with open("UnicodeData.txt") as f:
    diffs = []
    last_diff = None
    for l in f:
        if l[0] in ('#', '\n'):
            continue
        parts = l.split(";")
        if parts[2] != 'Ll' or len(parts[12]) == 0:
            continue
        lower_cp = int(parts[0], 16)
        upper_cp = int(parts[12], 16)
        diff = lower_cp - upper_cp
        if diff != last_diff:
            diffs.append((diff, []))
        if len(diffs[-1][1]) > 0 and lower_cp - diffs[-1][1][-1] != 1:
            diffs.append((diff, []))
        diffs[-1][1].append(lower_cp)
        last_diff = diff

    print_code(diffs)