const std = @import("std");
const mem = std.mem;
const fs = std.fs;
pub const program_name = "passphrase";

pub const Options = struct {
    dictionary: ?[]const u8 = null,
    separator: ?u8 = ' ',
    words: u8 = 4,
    letters: u8 = 32,
    capitalize: bool = false,
    add_symbol: bool = false,
    add_digit: bool = false,
    add_newline: bool = false,
    generate_password: bool = false,
    const Self = @This();

    pub fn parse(self: *Self, args: [][]const u8) !void {
        var i: usize = 0;

        loop: while (i < args.len) : (i += 1) {
            const arg = args[i];

            if (arg.len < 2 or arg[0] != '-')
                return error.InvalidOption;

            for (arg[1..]) |b, j| {
                switch (b) {
                    'w', 's', 'f', 'l' => {
                        i += 1;

                        if (j != arg.len - 2) {
                            try self.parseUnaryArg(b, arg[2 + j ..]);
                            continue :loop;
                        } else if (i < args.len) {
                            try self.parseUnaryArg(b, args[i]);
                        } else return error.MissingValue;
                    },
                    '-' => {
                        try self.parseLongArg(arg[2..]);
                        continue :loop;
                    },
                    else => try self.parseNullaryArg(b),
                }
            }
        }
    }

    fn parseNullaryArg(self: *Self, arg: u8) !void {
        switch (arg) {
            'c' => self.capitalize = true,
            'x' => self.add_symbol = true,
            'd' => self.add_digit = true,
            'p' => self.generate_password = true,
            'h' => return error.Help,
            else => return error.InvalidOption,
        }
    }

    fn parseUnaryArg(self: *Self, arg: u8, value: []const u8) !void {
        switch (arg) {
            'w' => {
                self.words = try std.fmt.parseInt(u8, value, 10);
            },
            'l' => {
                self.letters = try std.fmt.parseInt(u8, value, 10);
            },
            's' => {
                switch (value.len) {
                    0 => self.separator = null,
                    1 => self.separator = value[0],
                    else => return error.InvalidValue,
                }
            },
            'f' => {
                self.dictionary = value;
            },
            else => unreachable,
        }
    }

    fn parseLongArg(self: Self, arg: []const u8) !void {
        _ = self;
        if (mem.eql(u8, arg, "help"))
            return error.Help;

        return error.InvalidOption;
    }
};

pub const help_text = "Usage: " ++ program_name ++ " [options]\n" ++
    \\
    \\Options:
    \\  -w num    Use num words for passphrase [default: 4]
    \\  -s sep    Put a sep character (or '' for none) between the words [default: -]
    \\  -c        Capitalize each word
    \\  -x        Put a random symbol at the end
    \\  -d        Put a random digit at the end
    \\  -f file   Use file as dictionary (one word per line)
    \\  -p        Generate a random password instead of a passphrase
    \\  -l num    Use num characters for password [default: 24]
    \\  -h --help Show help and quit
    \\
;
